@@ -18,10 +18,6 @@ if [ ! -f ${BINDIR}/rmmod ]; then
   echo "${BINDIR}/rmmod not existing"
   exit 1
 fi
-if [ ! -f ${MODULESDIR}/syscallmgr.o ]; then
-  echo "${MODULESDIR}/syscallmgr.o not existing"
-  exit 1
-fi      
 if [ ! -f ${MODULESDIR}/emumodule.o ]; then
   echo "${MODULESDIR}/emumodule.o not existing"
   exit 1
@@ -47,8 +43,5 @@ fi
 if [ "`lsmod|grep emumodule`" != "" ]; then
   ${BINDIR}/rmmod emumodule
 fi
-if [ "`lsmod|grep syscallmgr`" = "" ]; then
-  ${BINDIR}/insmod -z ${MODULESDIR}/syscallmgr.o
-fi
  
 ${BINDIR}/insmod -lz ${MODULESDIR}/emumodule.o
