@@ -1,23 +1,9 @@
 #ifndef _EMUMOD_
 #define _EMUMOD_
 
-#if 1
-#define _VM86_STATISTICS_
-#endif
-
 #define ID_STRING "vm86-module"
 
-#ifdef _EMUMOD_itself
-
-#ifdef _VM86_STATISTICS_
-int vm86_fault_count=0;
-int vm86_trap_count[8]={0};
-int vm86_count_sti=0;
-int vm86_count_cli=0;  
-int signalret_count=0;
-#endif
-
-#else  /* NOT _EMUMOD_itself */
+#ifndef _EMUMOD_itself
 
 /* redirection for traps.c */
 #define do_divide_error _TRANSIENT_do_divide_error
@@ -34,10 +20,10 @@ asmlinkage struct pt_regs * _TRANSIENT_save_v86_state(struct vm86_regs * regs);
 void _TRANSIENT_handle_vm86_fault(struct vm86_regs * regs, long error_code);
 
 #define handle_vm86_trap _TRANSIENT_handle_vm86_trap
-void handle_vm86_trap(struct vm86_regs * regs, long error_code, int trapno);
+int handle_vm86_trap(struct vm86_regs * regs, long error_code, int trapno);
 
 #define sys_vm86 _TRANSIENT_sys_vm86
-asmlinkage int _TRANSIENT_sys_vm86(struct vm86_struct * v86);
+asmlinkage int _TRANSIENT_sys_vm86(unsigned long subfunction, struct vm86_struct * v86_);
 
 #define sys_sigreturn  _TRANSIENT_sys_sigreturn
 asmlinkage int _TRANSIENT_sys_sigreturn(unsigned long __unused);
@@ -45,6 +31,6 @@ asmlinkage int _TRANSIENT_sys_sigreturn(unsigned long __unused);
 #define sys_modify_ldt  _TRANSIENT_sys_modify_ldt
 asmlinkage int _TRANSIENT_sys_modify_ldt(int func, void *ptr, unsigned long bytecount);
 
-#endif /* _EMUMOD_itself */
+#endif /* not _EMUMOD_itself */
 
 #endif /* _EMUMOD_ */
